//******************************************
// file			: MQ6801_ADC.c
// version		: V1.1 2021/02/24
// brief		: ADC related functions
// note			: ADC related functions are gathered in this subroutine
//******************************************

#include "main.h"



uint  vref_l, vref_h, iReal_Vref2V, ADC_Code_READ;
ulong VrefReal2048;

//*********************************************************************
// name			: ADC_Init
// brief		: ADC initialization
// note			:
//*********************************************************************
void ADC_Init(uchar SAIN)
{  
//  MQ6801 AIN0~AIN8  analog input channel-----------

    ADCVRF_VRFSEL   = Vref2;            	// set ADC reference voltage to internal 2V
    ADCCR1_SAIN     = SAIN;             	// set analog input channel
    
    ADCCR1_AINEN    = 1;                	// analog input enable
    ADCCR1_AMD      = ADC_Repeat;       	// set AD operating mode to repeat mode
    
    
    
    ADCCR2_ACK      = Conv256CLK;       	// AD conversion time
}


//*********************************************************************
// name			: ADC_Start
// brief		: start ADC
// note			:
//*********************************************************************
void ADC_Start(void)
{
    ADCCR1_ADRS = 1;                    	
}


//*********************************************************************
// name			: ADC_Stop
// brief		: stop ADC	
// note			:
//*********************************************************************
void ADC_Stop(void)
{
    ADCCR1_AMD = 0;                     	// 0: AD operation disable, forcibly stop AD operation
}


//*********************************************************************
// name			: ADC_IntEnable
// brief		: enable ADC interrupt
// note			: after execution, Interrupt Master Allowed Flag (IMF) will be enabled
//*********************************************************************
void ADC_IntEnable(void)
{
    DI;                                 	// disable all maskable interrupts // EIRL_IMF=0
    IL_INTADC = 0;                      	// clear interrupt latch
    EI_INTADC = 1;                      	// set interrupt enable flag
    EI;                                 	// enable all maskable interrupts // EIRL_IMF=1
}


//*********************************************************************
// name			: ADC_IntDisable
// brief		: disable ADC interrupt
// note			: after execution, Interrupt Master Allowed Flag (IMF) will be enabled
//*********************************************************************
void ADC_IntDisable(void)
{
    DI;                                 	// disable all maskable interrupts // EIRL_IMF=0
    
    
    IL_INTADC = 0;                      	// clear interrupt latch
    EI_INTADC = 0;                      	// clear interrupt enable flag
    EI;                                 	// enable all maskable interrupts // EIRL_IMF=1
}


//*********************************************************************
// name			: GetVref()
// brief		: get the value of internal ADC reference voltage (unit: mV)
// note			:
//*********************************************************************
uint GetVref(void)
{
    BIT_LIST_16 ADC_READ;
//	Vref2V, value is ( Real_2V * 1024 )
	__asm("LD A,(0x7E05)");		        	// low byte
	__asm("LD (_vref_l),A");
	__asm("LD A,(0x7E06)");		        	// high byte
	__asm("LD (_vref_h),A");

	ADC_READ.byte[0] = vref_l;
	ADC_READ.byte[1] = vref_h & 0x0f;

	if( ADC_READ.word > 2100 )	ADC_READ.word = 2000;	// if Vref is out of range, do not use this value
	if( ADC_READ.word < 1900 )	ADC_READ.word = 2000;	// if Vref is out of range, do not use this value
	VrefReal2048 = (ulong)ADC_READ.word * 2048 / 2000;

    return ADC_READ.word;
}
//uint GetVref(void)
//{
////	__asm("LD A,(0x7E05)");		    	// low byte
////	__asm("LD A,(0x7E06)");		    	// high byte
//	uint *adc_value = (uint*)(0x7E05);
//	uint adc_read;

//  adc_read = *adc_value & 0x0fff;
//  if internal voltage value is between 2150 and 1950, start ADC auto-compensation.
//  if( ADC_READ.word <= 2150 )
//      if( ADC_READ.word >= 1950 )
//          VREF_CALIB_VRFCAL = 1;
//	return adc_read;
//}

//*********************************************************************
// name			: ADC_single_detect
// brief		: get ADC value of single coversion time
// note			:
//*********************************************************************
uint ADC_single_detect(uchar SAIN)
{
    BIT_LIST_16 ADC_READ;

//====== select analog input channel ======
    ADCCR1_SAIN = SAIN;                 	// set analog input channel

    ADCCR1_ADRS = 1;                    	
    while(  ADCCR2_ADBF ){ CLR_WDT; }   	// 1: AD conversion being executed
    while( !ADCCR2_EOCF ){ CLR_WDT; }   	// 1: conversion end  0: before conversion or during conversion  [EOCF:ADתɱ]

    ADC_READ.byte[0] = ADCDRL;          	// read low byte first and then read high byte
    ADC_READ.byte[1] = ADCDRH;          	// read low byte first and then read high byte
//    adc_data += ADC_READ.word;








	return ADC_READ.word;
}


//*********************************************************************
// name			: ADC_repeat_detect
// brief		: get ADC value after more times conversion
// note			:
//*********************************************************************
uint ADC_repeat_detect(uchar SAIN)
{
    uchar i;
    ulong adc_data = 0;
    uint  ad_max = 0, ad_min = 1024;
    BIT_LIST_16 ADC_READ;

//====== select analog input channel ======
    ADCCR1_SAIN = SAIN;                 	// set analog input channel

    ADCCR1_ADRS = 1;                    	
    for( i=0; i<(ad_test_times+2); i++ )	// repeat sampling 18 times
    {
        while( !ADCCR2_EOCF ){ CLR_WDT; } 	// 1: conversion end  0: before conversion or during conversion
        ADC_READ.byte[0] = ADCDRL;      	// read low byte first and then read high byte
        ADC_READ.byte[1] = ADCDRH;      	// read low byte first and then read high byte
        adc_data += ADC_READ.word;
        if( ADC_READ.word > ad_max ) ad_max = ADC_READ.word;
        if( ADC_READ.word < ad_min ) ad_min = ADC_READ.word;
    }
    ADCCR1_ADRS = 0;                    	// AD conversion stop

    adc_data -= ad_max;                 	// subtract the maximum value during sampling
    adc_data -= ad_min;                 	// subtract the minimum value during sampling
    adc_data /= ad_test_times;

	return (uint)adc_data;
}



//*********************************************************************
// name			: IntADC
// brief		: interrupt service routine
// note			:
//*********************************************************************
void __interrupt IntADC(void)
{
    NOP;                                	// no operation
}




























