//******************************************************************************
// file         : ADM_main.c
// version      : V1.3  2023/11/20
// description  : at the beginning of the program, LED1 (P2.0) will be the light off.
//                1. press button 1 BTN1(P1.0), external interrupt(EINT0) light on LED1
//                2. press button 2 BTN2(P1.1), external interrupt(EINT1) light off LED1
// note         : [Button Configuration]
//                       button 1 :  BTN1(P1.0) external interrupt(EINT0)IntEXT0
//                       button 2 :  BTN2(P1.1) external interrupt(EINT1)IntEXT1
//               [LED Configuration]    LED1  (P2.0)
//******************************************************************************
#ifndef IO_MEM
#define IO_MEM
#endif
#include "ADM_main.h"
#include "DRV_SQ7705_SYSCLK.h"
#include "DRV_SQ7705_GPIO.h"
#include "DRV_SQ7705_EINT.h"

/* --------------------------- global variables ----------------------------- */


/* ---------------------------- main program -------------------------------- */
void ADM_Main()
{
    DRV_Clock_Init(Clk_Source_SYS, 1);                  // PLL 24MHz Initial system clock.

    DRV_GPIO_Init(GPIO_P20, GPIO_OUTPUT, GPIO_PULL_UP); // set P2.0 as output pin
                                                        // enable P2.0 built-in pull-up resistor
    DRV_GPIO_WritePin(GPIO_P20, GPIO_PIN_HIGH);         // set LED1(P2.0) to light off
    P1PU |= 0x03;                                       // enable P1.0 and P1.1 built-in pull-up resistors

    /* external interrupt initializationEINT0 P10 an interrupt request is generated at the falling edge*/
    DRV_EINT_Init(EINT0_P10, INT_VL_HIGH, INTES_FALLING, FSYSCLK_DIV_1);
    DRV_EINT_Enable(0);
    /* external interrupt initializationEINT1 P11 an interrupt request is generated at the falling edge*/
    DRV_EINT_Init(EINT1_P11, INT_VL_HIGH, INTES_FALLING, FSYSCLK_DIV_1);
    DRV_EINT_Enable(1);

    //Push button 1 to ligth on LED1, push button light off LED 1
    while (1) {
        if (EXT0_Int_Flag) {                            // if external interrupt flag of button 1 = 1
            EXT0_Int_Flag = 0;                          // clear external interrupt flag to wait for the next press
            DRV_GPIO_WritePin(GPIO_P20, GPIO_PIN_LOW);  // light on LED1
        }

        if (EXT1_Int_Flag) {                            // if external interrupt flag of button 2 = 1
            EXT1_Int_Flag = 0;                          // clear external interrupt flag to wait for the next press
            DRV_GPIO_WritePin(GPIO_P20, GPIO_PIN_HIGH); // light off LED1
        }
    }
}

/* ----------------------- interrupt service routine ------------------------ */
void __interrupt OnlyReti(void)
{
    __ASM("NOP");                              // no operation
}

void __interrupt_n OnlyRetn(void)
{
    __ASM("NOP");                              // no operation
}

void __interrupt_n WDT_IRQ(void)
{
    __ASM("NOP");                              // no operation
}

void __interrupt_n SWI_IRQ(void)               // software reset interrupt service routine
{
    SWRST = 0x5a;
    SWRST = 0xa5;
    SWRST = 0xc3;
    SWRST = 0x3c;
}

void __interrupt IntADM_Req(void)
{

}

/* ---------------------- end of interrupt service routine ------------------ */


/* ----------------------- SQ7705 interrupt vector -------------------------- */
#pragma section const ADM_INT_VECTOR
void * const IntTbl1[] = {
    ADM_STARTUP,             /* 0x0000 : RESET               */    //non-maskable
    SWI_IRQ,                 /* 0x0002 : SWI_IRQ/INTUNDEF    */    //non-maskable
    (void *)0xffff,          /* 0x0004 : Reserved            */    
    WDT_IRQ,                 /* 0x0006 : WDT_IRQ             */    //non-maskable interrupt source:watchdog timer
    OnlyReti,                /* 0x0008 : TAMPER_IRQ          */    //interrupt source:tamper detection
    OnlyReti,                /* 0x000A : CFD_IRQ             */    //interrupt source:clock faildetection
    OnlyReti,                /* 0x000C : HVD_IRQ             */
    IntADM_Req,              /* 0x000E : U2A_IRQ/IntADM_Req  */    //interrupt source:user mode to admin mode
    OnlyReti,                /* 0x0010 : RTC_IRQ             */    //interrupt source:real-time clock timer
    OnlyReti,                /* 0x0012 : TMRA00_IRQ          */    //interrupt source:TMRA00 8-bit timer
    OnlyReti,                /* 0x0014 : TMRA01_IRQ          */    //interrupt source:TMRA01 8-bit timer
    OnlyReti,                /* 0x0016 : TMRA10_IRQ          */    //interrupt source:TMRA10 8-bit timer
    OnlyReti,                /* 0x0018 : TMRA11_IRQ          */    //interrupt source:TMRA11 8-bit timer
    OnlyReti,                /* 0x001A : TMRA20_IRQ          */
    OnlyReti,                /* 0x001C : TMRA21_IRQ          */
    OnlyReti,                /* 0x001E : TBT_IRQ             */
    OnlyReti,                /* 0x0020 : RX0_IRQ             */    //interrupt source:UART0 RX0
    OnlyReti,                /* 0x0022 : TX0_IRQ             */    //interrupt source:UART0 TX0
    OnlyReti,                /* 0x0024 : I2C0_IRQ            */    //interrupt source:I2C0
    OnlyReti,                /* 0x0026 : SPI0_IRQ            */    //interrupt source:SPI0
    DRV_EXT0_IRQ,            /* 0x0028 : EXT0_IRQ            */    //external interrupt0
    DRV_EXT1_IRQ,            /* 0x002A : EXT1_IRQ            */    //external interrupt1
    DRV_EXT2_IRQ,            /* 0x002C : EXT2_IRQ            */    //external interrupt2
    DRV_EXT3_IRQ,            /* 0x002E : EXT3_IRQ            */    //external interrupt3
    OnlyReti,                /* 0x0030 : ADC_IRQ             */    //interrupt source:ADC
    OnlyReti,                /* 0x0032 : LVD0_IRQ            */
    OnlyReti,                /* 0x0034 : LVD1_IRQ            */
    OnlyReti,                /* 0x0036 : Reserved            */
    OnlyReti,                /* 0x0038 : Reserved            */
    OnlyReti,                /* 0x003A : FMC_IRQ             */    //interrupt source:flash memory controller
    OnlyReti,                /* 0x003C : DMACH0_IRQ          */
    OnlyReti,                /* 0x003E : DMACH1_IRQ          */
    OnlyReti,                /* 0x0040 : DMACH2_IRQ          */
    OnlyReti,                /* 0x0042 : DMACH3_IRQ          */
    OnlyReti,                /* 0x0044 : Reserved            */
    OnlyReti,                /* 0x0046 : Reserved            */
    OnlyReti,                /* 0x0048 : Reserved            */
    OnlyReti,                /* 0x004A : Reserved            */
    OnlyReti,                /* 0x004C : Reserved            */
    OnlyReti,                /* 0x004E : DIC_IRQ             */    //interrupt source:DIC
    OnlyReti,                /* 0x0050 : LEUART_RX_IRQ       */
    OnlyReti,                /* 0x0052 : LEUART_TX_IRQ       */
    OnlyReti,                /* 0x0054 : TRNG_IRQ            */
    DRV_EXT4_IRQ,            /* 0x0056 : EXT4_IRQ            */    //external interrupt4
    DRV_EXT5_IRQ,            /* 0x0058 : EXT5_IRQ            */    //external interrupt5
    DRV_EXT6_IRQ,            /* 0x005A : EXT6_IRQ            */    //external interrupt6
    DRV_EXT7_IRQ,            /* 0x005C : EXT7_IRQ            */    //external interrupt7
    OnlyReti,                /* 0x005E : EEMC_IRQ            */    //interrupt source:EEPROM controller
    OnlyReti,                /* 0x0060 : TCA0_IRQ            */    //interrupt source:TCA0 16-bit timer
    OnlyReti,                /* 0x0062 : TCA1_IRQ            */    //interrupt source:TCA1 16-bit timer
    OnlyReti,                /* 0x0064 : STM0_IRQ            */
    OnlyReti,                /* 0x0066 : STM1_IRQ            */
    OnlyReti,                /* 0x0068 : AES_IRQ             */
    OnlyReti,                /* 0x006A : SHA_IRQ             */
    OnlyReti,                /* 0x006C : RX1_IRQ             */    //interrupt source:UART1 RX1
    OnlyReti,                /* 0x006E : TX1_IRQ             */    //interrupt source:UART1 TX1
    OnlyReti,                /* 0x0070 : I2C1_IRQ            */    //interrupt source:I2C1
    OnlyReti,                /* 0x0072 : SPI1_IRQ            */    //interrupt source:SPI1
    OnlyReti,                /* 0x0074 : RX2_IRQ             */    //interrupt source:UART2 RX2
    OnlyReti,                /* 0x0076 : TX2_IRQ             */    //interrupt source:UART2 TX2
    OnlyReti                 /* 0x0078 : TCA2_IRQ            */    //interrupt source:TCA2 16-bit timer
};
#pragma section const
/* -------------------- end of SQ7705 interrupt vector ---------------------- */

